﻿using Microsoft.Extensions.Logging;
using SelfPropelledArtilleryUnit.CollectionGenericObjects;
using SelfPropelledArtilleryUnit.Drawnings;
using SelfPropelledArtilleryUnit.Exceptions;


namespace SelfPropelledArtilleryUnit;

public partial class FormPropelledArtilleryCollection : Form
{
    /// <summary>
	/// Хранилише коллекций
	/// </summary>
	private readonly StorageCollection<DrawningPropelledArtillery> _storageCollection;

    /// <summary>
	/// Компания
	/// </summary>
	private AbstractCompany? _company = null;

    /// <summary>
	/// Логер
	/// </summary>
    private readonly ILogger _logger;

    /// <summary>
	/// Конструктор
	/// </summary>
    public FormPropelledArtilleryCollection(ILogger<FormPropelledArtilleryCollection> logger)
    {
        InitializeComponent();
        _storageCollection = new();
        _logger = logger;
        _logger.LogInformation("Форма загрузилась");
    }

    /// <summary>
    /// Выбор компании
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>

    private void ComboBoxSelectorCompany_SelectedIndexChanged(object sender, EventArgs e)
    {
        panelCompanyTools.Enabled = false;
    }

    private void buttonAddPropelledArtillery_Click(object sender, EventArgs e)
    {
        FormPropelledArtilleryConfig form = new();
        // TODO передать метод
        form.Show();
        form.AddEvent(SetPropelledArtillery);
    }

    /// <summary>
    /// Создание объекта класса-перемещения
    /// </summary>
    /// <param name="type">Тип создаваемого объекта</param>
    private void SetPropelledArtillery(DrawningPropelledArtillery propelledArtillery)
    {
        try
        {
            if (_company == null || propelledArtillery == null)
            {
                return;
            }
            if (_company + propelledArtillery != -1)
            {
                MessageBox.Show("Объект добавлен");
                pictureBox.Image = _company.Show();
                _logger.LogInformation("Добавлен объект: " + propelledArtillery.GetDataForSave());
            }
        }
        catch (ObjectNotFoundException) { }
        catch (CollectionOverflowException ex)
        {
            MessageBox.Show("Не удалось добавить объект");
            _logger.LogError("Ошибка: {Message}", ex.Message);
        }
        catch (ObjectIsEqualException ex)
        {
            MessageBox.Show("Не удалось добавить объект");
            _logger.LogError("Ошибка: {Message}", ex.Message);
        }
    }

    /// <summary>
	/// Удаление объекта
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
    private void buttonRemovePropelledArtillery_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(maskedTextBox.Text) || _company == null)
        {
            return;
        }
        if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
        {
            return;
        }
        int pos = Convert.ToInt32(maskedTextBox.Text);
        try
        {
            if (_company - pos != null)
            {
                MessageBox.Show("Объект удален");
                pictureBox.Image = _company.Show();
                _logger.LogInformation("Удален объект по позиции " + pos);
            }
        }
        catch (Exception ex)
        {
            MessageBox.Show("Не удалось удалить объект");
            _logger.LogError("Ошибка: {Message}", ex.Message);
        }
    }

    /// <summary>
    /// Передача объекта в другую форму
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonGoToCheck_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        DrawningPropelledArtillery? propelledartillery = null;
        int counter = 100;
        try
        {
            while (propelledartillery == null)
            {
                propelledartillery = _company.GetRandomObject();
                counter--;
                if (counter <= 0)
                {
                    break;
                }
            }
            SelfPropelledArtilleryUnit form = new()
            {
                SetPropelledArtillery = propelledartillery
            };
            form.ShowDialog();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    /// <summary>
    /// Перерисовка коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonRefresh_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }
        pictureBox.Image = _company.Show();
    }

    private void buttonColectionAdd_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(textBoxCollectionName.Text) || (!radioButtonList.Checked && !radioButtonMassive.Checked))
        {
            MessageBox.Show("Не все данные заполнены", "Ошибка",
            MessageBoxButtons.OK, MessageBoxIcon.Error);
            return;
        }
        try
        {
            CollectionType collectionType = CollectionType.None;
            if (radioButtonMassive.Checked)
            {
                collectionType = CollectionType.Massive;
            }
            else if (radioButtonList.Checked)
            {
                collectionType = CollectionType.List;
            }
            _storageCollection.AddCollection(textBoxCollectionName.Text, collectionType);
            RerfreshListBoxItems();
            _logger.LogInformation("Коллекция добавлена " + textBoxCollectionName.Text);
        }
        catch (Exception ex)
        {
            _logger.LogError("Ошибка: {Message}", ex.Message);
        }
    }
    private void RerfreshListBoxItems()
    {
        listBoxCollection.Items.Clear();
        for (int i = 0; i < _storageCollection.Keys?.Count; ++i)
        {
            string? colName = _storageCollection.Keys?[i].Name;
            if (!string.IsNullOrEmpty(colName))
            {
                listBoxCollection.Items.Add(colName);
            }
        }
    }
    private void buttonCollectionDel_Click(object sender, EventArgs e)
    {
        // TODO прописать логику удаления элемента из коллекции
        // нужно убедиться, что есть выбранная коллекция
        // спросить у пользователя через MessageBox, что он подтверждает, что хочет удалить запись
        // удалить и обновить ListBox
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            return;
        }
        try
        {
            if (MessageBox.Show("Удалить коллекцию?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            _storageCollection.DelCollection(listBoxCollection.SelectedItem.ToString());
            RerfreshListBoxItems();
            _logger.LogInformation("Коллекция: " + listBoxCollection.SelectedItem.ToString() + " удалена");
        }
        catch (Exception ex)
        {
            _logger.LogError("Ошибка: {Message}", ex.Message);
        }
    }

    private void buttonCreateCompany_Click(object sender, EventArgs e)
    {
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            return;
        }
        ICollectionGenericObjects<DrawningPropelledArtillery>? collection =
            _storageCollection[listBoxCollection.SelectedItem.ToString() ?? string.Empty];
        if (collection == null)
        {
            MessageBox.Show("Коллекция не проинициализирована");
            return;
        }
        switch (comboBoxSelectorCompany.Text)
        {
            case "Хранилище":
                _company = new ArtilleryBase(pictureBox.Width, pictureBox.Height, collection);
                break;
        }
        panelCompanyTools.Enabled = true;
        RerfreshListBoxItems();
    }

    private void saveToolStripMenuItem_Click(object sender, EventArgs e)
    {
        if (saveFileDialog.ShowDialog() == DialogResult.OK)
        {
            try
            {
                _storageCollection.SaveData(saveFileDialog.FileName);
                MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                _logger.LogInformation("Сохранение в файл: {filename}", saveFileDialog.FileName);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
        }
    }

    private void loadToolStripMenuItem_Click(object sender, EventArgs e)
    {
        if (openFileDialog.ShowDialog() == DialogResult.OK)
        {
            try
            {
                _storageCollection.LoadData(openFileDialog.FileName);
                MessageBox.Show("Загрузка прошла успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                RerfreshListBoxItems();
                _logger.LogInformation("Загрузка из файла: {filename}", openFileDialog.FileName);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Не сохранилось", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
        }
    }

    private void buttonSortByType_Click(object sender, EventArgs e)
    {
        ComparePropelledArtillery(new DrawningPropelledArtilleryCompareByType());

    }

    private void ComparePropelledArtillery(IComparer<DrawningPropelledArtillery?> comparer)
    {
        if (_company == null)
        {
            return;
        }
        _company.Sort(comparer);
        pictureBox.Image = _company.Show();
    }

    private void buttonSortByColor_Click(object sender, EventArgs e)
    {
        ComparePropelledArtillery(new DrawningPropelledArtilleryCompareByColor());
    }
}


