﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using SelfPropelledArtilleryUnit.Drawnings;
using SelfPropelledArtilleryUnit.MovementStrategy;

namespace SelfPropelledArtilleryUnit
{
    public partial class SelfPropelledArtilleryUnit : Form
    {
        private DrawningPropelledArtillery? _drawningPropelledArtillery;

        /// <summary>
        /// Стратегия перемещения
        /// </summary>
        private AbstractStrategy? _strategy;

        /// <summary>
        /// Получение объекта 
        /// </summary>
        public DrawningPropelledArtillery SetPropelledArtillery
        {
            set
            {
                _drawningPropelledArtillery = value;
                _drawningPropelledArtillery.SetPictureSize(pictureBoxSelfPropelledArtilleryUnit.Width, pictureBoxSelfPropelledArtilleryUnit.Height);
                comboBoxStrategy.Enabled = true;
                _strategy = null;
                Draw();
            }
        }

        /// <summary>
        /// Конструктор формы
        /// </summary>
        public SelfPropelledArtilleryUnit()
        {
            InitializeComponent();
            _strategy = null;
        }

        private void Draw()
        {
            if (_drawningPropelledArtillery == null)
            {
                return;
            }

            Bitmap bmp = new(pictureBoxSelfPropelledArtilleryUnit.Width, pictureBoxSelfPropelledArtilleryUnit.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawningPropelledArtillery.DrawTransport(gr);
            pictureBoxSelfPropelledArtilleryUnit.Image = bmp;
        }

        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (_drawningPropelledArtillery == null)
            {
                return;
            }

            string name = ((Button)sender)?.Name ?? string.Empty;
            bool result = false;
            switch (name)
            {
                case "buttonUp":
                    result = _drawningPropelledArtillery.MoveTransport(DirectionType.Up); break;
                case "buttonDown":
                    result = _drawningPropelledArtillery.MoveTransport(DirectionType.Down); break;
                case "buttonLeft":
                    result = _drawningPropelledArtillery.MoveTransport(DirectionType.Left); break;
                case "buttonRight":
                    result = _drawningPropelledArtillery.MoveTransport(DirectionType.Right); break;
            }

            if (result)
            {
                Draw();
            }
        }

        /// <summary>
        /// Обработка нажатия кнопки "Шаг"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonStrategyStep_Click(object sender, EventArgs e)
        {
            if (_drawningPropelledArtillery == null)
            {
                return;
            }

            if (comboBoxStrategy.Enabled)
            {
                _strategy = comboBoxStrategy.SelectedIndex switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null,
                };
                if (_strategy == null)
                {
                    return;
                }
                _strategy.SetData(new MoveablePropelledArtillery(_drawningPropelledArtillery), pictureBoxSelfPropelledArtilleryUnit.Width, pictureBoxSelfPropelledArtilleryUnit.Height);
            }

            if (_strategy == null)
            {
                return;
            }

            comboBoxStrategy.Enabled = false;
            _strategy.MakeStep();
            Draw();

            if (_strategy.GetStatus() == StrategyStatus.Finish)
            {
                comboBoxStrategy.Enabled = true;
                _strategy = null;
            }
        }

        private void comboBoxStrategy_SelectedIndexChanged(object sender, EventArgs e)
        {

        }
    }
}
